<?php

namespace EasyContent\WordPress;

use Doctrine\Common\Annotations\AnnotationReader;
use EasyContent\WordPress\Controllers\Main;
use EasyContent\WordPress\Db\DbController;
use EasyContent\WordPress\Metaboxes\MetaBoxesManager;
use EasyContent\WordPress\Pages\PagesManager;
use EasyContent\WordPress\Rest\Actions\GetArticleDetails;
use EasyContent\WordPress\Rest\Actions\GetWorkflowStages;
use EasyContent\WordPress\Rest\Actions\PublishArticle;
use EasyContent\WordPress\Rest\Actions\PullArticle;
use EasyContent\WordPress\Rest\Actions\PushPost;
use EasyContent\WordPress\Rest\Actions\SetArticleWorkflowStage;
use EasyContent\WordPress\Rest\Actions\UnlinkPost;
use EasyContent\WordPress\Rest\RestManager;
use EasyContent\WordPress\Storages\ImageProcessingSettings;
use EasyContent\WordPress\Storages\PluginSettings;
use EasyContent\WordPress\Storages\PluginMisc;
use EasyContent\WordPress\Storages\StorageManager;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use Symfony\Component\Form\Extension\HttpFoundation\HttpFoundationExtension;
use Symfony\Component\Form\Extension\Validator\ValidatorExtension;
use Symfony\Component\Form\FormFactory;
use Symfony\Component\Form\Forms;
use Symfony\Component\Validator\Validation;

final class Plugin extends Singleton
{
    /** @var NoticesManager $noticesManager */
    public $noticesManager;

    /** @var DbController $dbController */
    private $dbController;

    /** @var bool $initialized */
    private $initialized = false;

    /** @var StorageManager $storageManager */
    private $storageManager;

    /** @var FormFactory $formFactory */
    private $formFactory;

    /** @var Logger $logger */
    private $logger;


    public function init()
    {
        if ( $this->initialized ) {
            return;
        }

        $this->noticesManager = new NoticesManager();
        $this->storageManager = new StorageManager( [
            'settings' => new PluginSettings(),
            'misc' => new PluginMisc(),
            'image-processing' => new ImageProcessingSettings()
        ] );

        $mainController = new Main();
        new PagesManager( [$mainController, 'renderAction'] );

        new RestManager( RestManager::EASYCONTENT_V1_NAMESPACE, [
            '/posts'                             => PublishArticle::class, // POST
            '/stages'                            => GetWorkflowStages::class, // GET
            '/posts/(?P<post_id>\d+)'            => SetArticleWorkflowStage::class, // PATCH
            '/posts/(?P<post_id>\d+)/push'       => PushPost::class, // POST
            '/posts/(?P<post_id>\d+)/unlink'     => UnlinkPost::class, // POST
            '/posts/(?P<post_id>\d+)/article'    => GetArticleDetails::class, // GET
            '/articles/(?P<article_id>\d+)/pull' => PullArticle::class // POST
        ] );
        new ListTableManager();

        new MetaBoxesManager();



        add_action( 'wp_loaded', [$this->getDatabaseController(), 'checkDbVersion'] );

        register_activation_hook( EASYCONTENT_ENTRY_FILE, [$this, 'onActivate'] );
        register_deactivation_hook( EASYCONTENT_ENTRY_FILE, [$this, 'onDeactivate'] );
        register_uninstall_hook( EASYCONTENT_ENTRY_FILE, [static::class, 'onUninstall'] );
        add_action( 'upgrader_process_complete', [$this, 'onUpgradeComplete'], 10, 2 );

        add_action( 'init', [$this, 'loadTextDomain'] );

        add_action( 'admin_enqueue_scripts', [$this, 'enqueueAdminAssets'] );

        add_action( 'delete_post', [Post::class, 'onDelete'], 10, 1 );
        add_action( 'save_post', [Post::class, 'onSave'], 10000, 3 );



        add_action( 'init', function() {
            wp_register_style(
                'ec-gutenberg',
                EASYCONTENT_URL . 'assets/css/gutenberg.css'
            );

            wp_register_script(
                'ec-gutenberg',
                EASYCONTENT_URL . 'assets/js/gutenberg/build/index.js',
                ['wp-plugins', 'wp-edit-post', 'wp-element']
            );
        } );

        add_action( 'enqueue_block_editor_assets', function() {
            wp_enqueue_style( 'ec-gutenberg' );
            wp_enqueue_script( 'ec-gutenberg' );
        } );

        add_filter( 'is_protected_meta', function($protected, $metaKey, $metaType) {
            $allowed = ['_aioseop_title', '_aioseop_description', '_yoast_wpseo_title', '_yoast_wpseo_metadesc'];

            return in_array( $metaKey, $allowed ) ? false : $protected;
        }, 10000, 3 );

        // This filter allows us to insert post with empty content and insert modified data after that
        add_filter( 'wp_insert_post_empty_content', function($maybeEmpty, $postArray) {
            return false;
        }, 10, 2 );

        $this->initialized = true;
    }


    public function loadTextDomain()
    {
        $path = 'easycontent-wp/languages';
        load_plugin_textdomain( EASYCONTENT_TXTDOMAIN, false, $path );
    }


    public function enqueueAdminAssets($hookSuffix)
    {
        wp_enqueue_style( 'ec-style', EASYCONTENT_URL . 'assets/css/plugin.css', [], EASYCONTENT_PLUGIN_VERSION );

        if ( 'easycontent_page_easycontent-publishing-options' === $hookSuffix
            || 'post.php' === $hookSuffix
            || 'post-new.php' === $hookSuffix
        ) {
            wp_enqueue_style( 'ec-jquery-ui', EASYCONTENT_URL . 'assets/css/vendor/jquery-ui/jquery-ui.min.css', [], EASYCONTENT_PLUGIN_VERSION );
            wp_enqueue_script( 'ec-admin', EASYCONTENT_URL . 'assets/js/admin.js', ['jquery', 'jquery-ui-dialog'], EASYCONTENT_PLUGIN_VERSION );
        }
    }


    public function getFormFactory()
    {
        if ( ! $this->formFactory ) {
            $this->formFactory = Forms::createFormFactoryBuilder()
                ->addExtensions( [
                    new HttpFoundationExtension(),
                    new ValidatorExtension(
                        Validation::createValidatorBuilder()
                            ->enableAnnotationMapping(new AnnotationReader())
                            ->getValidator()
                    )
                ] )
                ->getFormFactory();
        }

        return $this->formFactory;
    }


    /**
     * @return PluginSettings
     * @throws \Exception
     */
    public function getSettings()
    {
        return $this->storageManager->getStorage( 'settings' );
    }


    /**
     * @return PluginMisc
     * @throws \Exception
     */
    public function getMisc()
    {
        return $this->storageManager->getStorage( 'misc' );
    }


    /**
     * @return ImageProcessingSettings
     * @throws \Exception
     */
    public function getImageProcessingSettings()
    {
        return $this->storageManager->getStorage( 'image-processing' );
    }


    public function onActivate()
    {
        $this->getDatabaseController()->migrate();
    }


    public function onUpgradeComplete(\WP_Upgrader $upgrader, $hookExtra)
    {
        if ( 'plugin' !== $hookExtra['type'] ) {
            return;
        }

        if ( ! in_array( 'easycontent/easycontent-wp.php', $hookExtra['plugins'], true ) ) {
            return;
        }

         $this->getDatabaseController()->migrate();
    }


    public function onDeactivate()
    {
        if ( ! EASYCONTENT_DEBUG ) {
            return;
        }

        $this->getDatabaseController()->clearDatabase();
    }


    public static function onUninstall()
    {
        Plugin::getInstance()->getDatabaseController()->clearDatabase();
    }


    /**
     * @return DbController
     */
    public function getDatabaseController()
    {
        if ( ! $this->dbController ) {
            $this->dbController = new DbController();
        }

        return $this->dbController;
    }


    protected function getLogger()
    {
        if ( ! $this->logger ) {
            $this->logger = new Logger( 'easycontent-logger' );

            $logsDir = EASYCONTENT_DIR . DIRECTORY_SEPARATOR . 'logs';
            $logFileName = 'debug_' . date( 'Y-m-d' ) . '.txt';

            $this->logger->pushHandler( new StreamHandler( $logsDir . DIRECTORY_SEPARATOR . $logFileName ) );
        }

        return $this->logger;
    }


    public function log($message, $context = [])
    {
        if ( ! EASYCONTENT_DEBUG ) {
            return;
        }

        $this->getLogger()->log( Logger::DEBUG, $message, (array)$context );
    }


    public function getRefererString()
    {
        $scheme = isset( $_SERVER['REQUEST_SCHEME'] ) ? $_SERVER['REQUEST_SCHEME'] : 'http';
        $host = ! empty( $_SERVER['HTTP_HOST'] ) ? $_SERVER['HTTP_HOST'] : null;

        if ( null === $host ) {
            return null;
        }

        return "{$scheme}://{$host}";
    }
}
