<?php

namespace EasyContent\WordPress\Api;

use EasyContent\WordPress\Db\Models\Article;
use EasyContent\WordPress\Db\Models\Category;
use EasyContent\WordPress\Db\Models\WorkflowStage;
use EasyContent\WordPress\Exceptions\EasyContentResponseException;
use EasyContent\WordPress\Forms\Entities\ApiKeys;
use EasyContent\WordPress\Forms\Entities\ServerConnectionResponse;
use EasyContent\WordPress\Forms\Types\ArticleType;
use EasyContent\WordPress\Forms\Types\CategoryType;
use EasyContent\WordPress\Forms\Types\WorkflowStageType;
use EasyContent\WordPress\Plugin;
use GuzzleHttp\Exception\BadResponseException;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\Extension\Core\Type\FormType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\UrlType;

class Connector
{
    public function connectUsingApiKeys(ApiKeys $keys)
    {
        $plugin = Plugin::getInstance();

        $plugin->log( 'Connecting to EasyContent.io Rest API...' );

        $httpClient = Client::create( $keys->getPublic() );

        try {
            $response = $httpClient->request( 'POST', 'connect' );
        } catch ( BadResponseException $e ) {
            $ecException = new EasyContentResponseException( $e );
            throw $ecException->getMessage() ? new \Exception( $ecException->getMessage() ) : $e;
        }

        $responseData = json_decode( $response->getBody()->getContents(), true );

        $plugin->log( 'Request performed' );

        if ( $response->getStatusCode() != 200 ) {
            $message = isset( $responseData['message'] )
                ? $responseData['message']
                : __( 'Invalid key pair or internal server error', EASYCONTENT_TXTDOMAIN );

            throw new \Exception( $message );
        }

        $form = Plugin::getInstance()
            ->getFormFactory()
            ->createBuilder( FormType::class, new ServerConnectionResponse() )
            ->add( 'accountName', TextType::class )
            ->add( 'projectName', TextType::class )
            ->add( 'editUrlBase', UrlType::class )
            ->add( 'articles', CollectionType::class, [
                'entry_type' => ArticleType::class,
                'allow_add' => true
            ] )
            ->add( 'categories', CollectionType::class, [
                'entry_type' => CategoryType::class,
                'allow_add' => true
            ] )
            ->add( 'stages', CollectionType::class, [
                'entry_type' => WorkflowStageType::class,
                'allow_add' => true
            ] )
            ->getForm();

        $form->submit( $responseData );

        if ( ! $form->isSubmitted() || ! $form->isValid() ) {
            throw new \Exception( __( 'Data was not sent back or invalid', EASYCONTENT_TXTDOMAIN ) );
        }

        // Article::truncate();
        // Category::truncate();
        // WorkflowStage::truncate();
        // ArticlePost::truncate();

        $this->saveRetrievedData( $keys, $form->getData() );

        return $form->getData();
    }


    protected function saveRetrievedData(ApiKeys $keys, ServerConnectionResponse $model)
    {
        Plugin::getInstance()->log( 'Saving retrieved data' );

        $settings = Plugin::getInstance()->getSettings();
        $misc = Plugin::getInstance()->getMisc();

        $settings->publicKey  = $keys->getPublic();
        $settings->isPluginConnected = true;

        $misc->accountName = $model->getAccountName();
        $misc->projectName = $model->getProjectName();
        $misc->editUrlBase = $model->getEditUrlBase();

        Category::setDatabaseCache( $model->getCategories() );
        WorkflowStage::setDatabaseCache( $model->getStages() );

        $articles = array_map( function($article) {
            unset( $article->categories );
            return $article;
        }, $model->getArticles() );
        $articles = Article::setDatabaseCache( $articles );
        Article::updateRelatedData( $articles );
    }
}
