<?php

namespace EasyContent\WordPress\Storages;

/**
 * @property string $publicKey
 * @property bool $isPluginConnected
 *
 * @property integer $postAuthorId
 * @property string $postType
 * @property string $postStatus
 * @property string $commentsStatus
 * @property array $mappings
 * @property integer $defaultCategoryId
 *
 * @property null|string $seoPlugin
 */
class PluginSettings extends BaseOptionsStorage
{
    /** @var null|\WP_User $_user */
    protected $_user;


    protected function getDefaults()
    {
        return [
            'publicKey' => '',
            'isPluginConnected' => false,

            'postAuthorId' => 0,
            'postType' => 'post',
            'postStatus' => 'publish',
            'commentsStatus' => 'open',
            'mappings' => [],
            'defaultCategoryId' => (int)get_option( 'default_category' ),

            'seoPlugin' => null
        ];
    }


    protected function getMappings()
    {
        return [
            'publicKey' => [
                'name'   => 'ec-public-api-key',
                'filter' => 'strval'
            ],
            'isPluginConnected' => [
                'name'   => 'ec-is-plugin-connected',
                'filter' => 'boolval'
            ],

            'postAuthorId' => [
                'name'   => 'ec-post-author-id',
                'filter' => 'absint'
            ],
            'postType' => [
                'name'   => 'ec-post-type',
                'filter' => 'strval'
            ],
            'postStatus' => [
                'name'   => 'ec-post-status',
                'filter' => 'strval'
            ],
            'commentsStatus' => [
                'name' => 'ec-comments-status',
                'filter' => 'strval'
            ],
            'mappings' => [
                'name'   => 'ec-post-categories-mappings',
                'filter' => [static::class, 'toArray']
            ],
            'defaultCategoryId' => [
                'name'   => 'ec-default-post-category-id',
                'filter' => 'absint'
            ],

            'seoPlugin' => [
                'name'   => 'ec-selected-seo-plugin',
                'filter' => 'strval'
            ]
        ];
    }


    public function getPostAuthorId()
    {
        if ( null === $this->_user ) {
            $this->_user = get_user_by( 'id', $this->postAuthorId );
        }

        // User was deleted or something
        if ( ! $this->_user && $this->postAuthorId !== 0 ) {
            $this->postAuthorId = 0;
        }

        return $this->postAuthorId;
    }


    public function getSeoPlugin()
    {
        $pluginPathes = [
            'yoast-seo' => 'wordpress-seo/wp-seo.php',
            'all-in-one-seo-pack' => 'all-in-one-seo-pack/all_in_one_seo_pack.php'
        ];

        if ( ! function_exists( 'is_plugin_active' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }

        // WP has no data about user choice
        if ( ! $this->seoPlugin ) {
            // One of listed plugins was installed and activated

            if ( is_plugin_active( $pluginPathes['yoast-seo'] ) && ! is_plugin_active( $pluginPathes['all-in-one-seo-pack'] ) ) {
                return $this->seoPlugin = 'yoast-seo';
            }

            if ( ! is_plugin_active( $pluginPathes['yoast-seo'] ) && is_plugin_active( $pluginPathes['all-in-one-seo-pack'] ) ) {
                return $this->seoPlugin = 'all-in-one-seo-pack';
            }

            // None were activated or both were activated (user choice needed)
            return null;
        }

        // Option has invalid value
        if ( 'all-in-one-seo-pack' !== $this->seoPlugin && 'yoast-seo' !== $this->seoPlugin ) {
            return $this->seoPlugin = null;
        }

        $secondPlugin = 'yoast-seo' === $this->seoPlugin ? 'all-in-one-seo-pack' : 'yoast-seo';

        if ( ! is_plugin_active( $pluginPathes[$this->seoPlugin] ) ) {
            $this->seoPlugin = is_plugin_active( $pluginPathes[$secondPlugin] )
                ? $secondPlugin
                : null;
        }

        return $this->seoPlugin;
    }


    public static function toArray($value)
    {
        return (array)$value;
    }
}
